﻿Imports System.ComponentModel
Imports System.ComponentModel.DataAnnotations
' ReSharper disable RedundantUsingDirective
Imports IdeaBlade.Core
' ReSharper restore RedundantUsingDirective
Imports IdeaBlade.Validation

' ReSharper disable RedundantNameQualifier
' ReSharper disable InconsistentNaming


<MetadataType(GetType(Customer.CustomerMetadata)), DebuggerDisplay("{ToString()}")>
Partial Public Class Customer
    Public Sub New()
    CustomerID = System.Guid.NewGuid()
    End Sub

    <Display(Name:="Original", AutoGenerateField:=True, Order:=3)>
    Public ReadOnly Property IsOriginal() As Boolean
        Get
            Return CustomerID_OLD <> String.Empty
        End Get
    End Property

    Public ReadOnly Property CanDelete() As Boolean
        Get
            Return Not IsOriginal
        End Get
    End Property

    ' Enforce integrity of order before adding it
    Public Sub AddOrder(ByVal order As Order)
        ' To Do: something real
        order.Customer = Me
    End Sub

    ' Enforce rules for removing an order
    Public Sub RemoveOrder(ByVal order As Order)
        Throw New InvalidOperationException("Can't remove an order")
    End Sub

    Public Overrides Function ToString() As String
        Return String.Format("{0}({1})", CompanyName, CustomerID)
    End Function

    ' Trim spaces from Company name
    <BeforeSet(EntityPropertyNames.CompanyName)>
    Public Sub TrimNameBeforeSet(ByVal args As PropertyInterceptorArgs(Of Customer, String))
        If args.Value IsNot Nothing Then
            args.Value = args.Value.Trim()
        End If
    End Sub

#Region "Customer Metadata"

    Public NotInheritable Class CustomerMetadata
        <Display(Name:="ID", AutoGenerateField:=True, Order:=0), Editable(False)>
        Public Shared CustomerID As Guid

        <Display(Name:="Name", AutoGenerateField:=True, Order:=1), DoNothingCustom()>
        Public Shared CompanyName As String

        ' Country is nullable in database but we want to require it
        <RequiredValueVerifier()>
        Public Shared Country As String

        <Bindable(True, BindingDirection.OneWay), Editable(False), Display(Name:="CustomerID_OLD", AutoGenerateField:=False)>
        Public Shared CustomerID_OLD As Guid
    End Class

#End Region
End Class

Public Class DoNothingCustomAttribute
    Inherits Attribute
End Class

